{*******************************************************************}
{  String and Other Misc. Functions for XCL.  Some are written in   }
{  ASM for extreme speed and small size.                            }
{            by Howard Flank, Flank Associates  flank@earthling.net }
{*******************************************************************}
{$H+,R-.I+}
unit HfUtils;
interface
uses Windows;
  Function  IIf(Cond:Boolean; const TrueVar,FalseVar:Variant): Variant;
  Function  StrIf(Cond:Boolean; const TrueStr,FalseStr:String): String;
  Function  IntIf(Cond:Boolean; TrueInt,FalseInt:Integer): Integer;
  Function  Max(Int1,Int2:Integer): Integer;
  Function  Min(Int1,Int2:Integer): Integer;
  Procedure AppendSStr(var Strng:ShortString; const App:ShortString);
  Function  ExecAndWait(CmdLine,StartDir:ShortString; WinShow:Integer): DWord;
  Function  StrLen(PStr: PChar): Integer;
  Function  Lookup(const Strng,Item:String; Len:Integer): Integer;
  Function  Scan(const Strng,Brk: String; StPosn:Integer;
            var Which:Integer): Integer;
  Function  ScanChar(const Strng:String; OneCh:Char; StPosn:Integer): Integer;
implementation
const  XTbl: Array[0..255] of Char = ' ';

{-----------------------------------------------------------------}
{  In-Line 'IF' Routines Returns First Variant/String/Integer if  }
{     <Cond> is True, Second if <Cond> is False.                  }
{  Example:  Msg := IIf(n=0,'No',IntToStr(n))+' Errors';          }
{  Replaces:  If n=0 Then Msg := 'No Errors'                      }
{             Else Msg := IntToStr(n)+' Errors';                  }
{-----------------------------------------------------------------}
Function IIf(Cond:Boolean; const TrueVar,FalseVar:Variant): Variant;  begin
  If Cond Then Result := TrueVar  Else Result := FalseVar;   end;
Function StrIf(Cond:Boolean; const TrueStr,FalseStr:String): String;  begin
  If Cond Then Result := TrueStr  Else Result := FalseStr;   end;
Function IntIf(Cond:Boolean; TrueInt,FalseInt:Integer): Integer;  begin
  If Cond Then Result := TrueInt  Else Result := FalseInt;   end;


{---------------------------------------------------------------}
{  Returns Maximum/Minimum of Two Integers.  Usage is Obvious.  }
{---------------------------------------------------------------}
Function Max(Int1,Int2:Integer): Integer;  begin
  If Int1>=Int2 Then Result := Int1  Else Result := Int2;   end;
Function Min(Int1,Int2:Integer): Integer;  begin
  If Int1<=Int2 Then Result := Int1  Else Result := Int2;   end;


{---------------------------------------------------------------------------}
{  AppendStr for ShortStrings, Since Delphi 2+ AppendStr doesn't like them  }
{---------------------------------------------------------------------------}
Procedure AppendSStr(var Strng:ShortString; const App:ShortString);
  var LS,Len: Integer;  begin
  LS := Length(Strng);  Len := Min(Length(App),High(Strng)-LS);
  If Len>0 Then begin  Move(App[1],Strng[LS+1],Len);
    Strng[0] := Char(LS+Len);   end;
  end{AppendSStr};


{------------------------------------------------------------------}
{  Execute a Win16, Win32 or DOS Program and Wait for Completion   }
{  Can specify Startup Directory (Use '.' for default directory)   }
{  WinShow Values:  sw_Hide =0; sw_ShowNormal or sw_Normal =1,     }
{      sw_ShowMinimized =2, sw_ShowMaximized =3, sw_ShowNA =8,     }
{      sw_ShowNoActivate =4, sw_ShowMinNoActive =7                 }
{  Note: Add $100 to WinShow to skip wait and return immediately.  }
{  Returns: Program Exit Code -OR- Process Handle if no wait -OR-  }
{      Negative (Error Code) if ExeAndWait Fails                   }
{  See CreateProcess, ShowWindow and Error Codes in Win32.Hlp.     }
{------------------------------------------------------------------}
Function ExecAndWait(CmdLine,StartDir:ShortString; WinShow:Integer): DWord;
  var  StartInf:TStartupInfo;  ProcessInf:TProcessInformation;
begin
  FillChar(StartInf,Sizeof(StartInf),#0);
  StartInf.cb := Sizeof(StartInf);
  StartInf.dwFlags := StartF_UseShowWindow;
  StartInf.wShowWindow := WinShow and $F;
  CmdLine[Length(CmdLine)] := #0;  StartDir[Length(StartDir)] := #0;
  {Syntax: CreateProcess(PgmName, CmdLine, ProcSecurityAttrs,
     ThreadSecurityAttrs, HandleInheritanceFlg, CreationFlgs,
     NewEnvironBlock, StartDir, StartupInfo, ProcessInfo);}
  If CreateProcess(nil, @CmdLine[1], nil, nil, false, Create_New_Console,
      nil, @StartDir[1], StartInf, ProcessInf) Then {}
    If WinShow<$100 Then begin {-Wait for Completion-}
      WaitforSingleObject(ProcessInf.hProcess,Infinite);
      GetExitCodeProcess(ProcessInf.hProcess,Result);   end{Wait}
    Else Result := Integer(ProcessInf.hProcess)
  Else Result := -GetLastError;   end{ExecAndWait};


{--------------------------------------------------}
{  Retruns the Length of a Null-Terminated String  }
{--------------------------------------------------}
Function StrLen(PStr: PChar): Integer; assembler;
  asm           Push    EDI
                Mov     EDI,PStr
                Mov     ECX,0FFFFFFFFh
                Xor     AL,AL
                RepNE   SCASB
                Not     ECX
                Mov     EAX,ECX
                Dec     EAX
                Pop     EDI
  end{StrLen};


{-----------------------------------------------------------------------}
{  LOOKUP an Item in a String of fixed length entries.  Faster then     }
{  using POS for this purpose, returns Item No. instead of char index   }
{  and will only match at start of an table entry.                      }
{    Strng - String containing a table of fixed length entries.         }
{    Item  - Item to be looked for in table (blank pad if < Len).       }
{    Len   - Length of each item in Strng.                              }
{      Rtns: Item No. [i.e. 1,2,3] if Item was found, 0 if not found.   }
{  Example: n := Lookup('alphabeta gamma',Item,5); will find 'alpha'=1, }
{      'beta'=2, and 'gamma'=3 but not find 'alph', 'gam' or 'phab'.    }
{-----------------------------------------------------------------------}
Function Lookup(const Strng,Item:String; Len:Integer): Integer;  Assembler;
  asm           Push    ESI             {Save ESI,EDI,EBX}
                Push    EDI
                Push    EBX
{---- Get String Pointers and Lengths, Get Item Length (Len)  ----}
                Mov     EDI,EAX         {EDI = @Strng}
                Or      EAX,EAX
                JZ      @NotFound       {Not Found if <Strng> is <nil>}
                Mov     EAX,[EAX-4]     {Len of <Strng>}
                Mov     ESI,EDX         {ESI = @Item}
                Or      EDX,EDX
                JZ      @NotFound       {Not Found if <Item> is <nil>}
                Mov     EDX,[EDX-4]     {Len of <Item>}
                Mov     EBX,ECX         {<Len> of each item in <Strng>}
                Mov     ECX,EDX         {CX = Len of <Item> to lookup}
                Mov     EDX,EAX         {DX = Len of <Strng> table}
                Add     EDX,EDI         {Addr just after end of <Strng>}
                Xor     EAX,EAX         {Reset entry counter}

{----  Compare Item to Table Entries  ----}
@Compare:       Inc     EAX             {Table entry counter}
                Push    EDI             {Save regs}
                Push    ESI
                Push    ECX
                RepZ    CmpSB           {Compare Item to Entry}
                Pop     ECX
                Pop     ESI
                JNE     @Next           {No match, try the next entry }

{---- Check Entry for trailing Blanks to Insure a Complete Match----}
@Match:         Push    ECX
                Sub     ECX,EBX         {Complete Match?}
                JE      @OK             {Yes if Len(Item) = Item <Len>}
                Neg     ECX             {Maybe: if rest of entry is blanks}
                Push    EAX
                Mov     AL," "
                RepZ    ScaSB           {Scan for end of entry for blanks}
                Pop     EAX

@OK:            Pop     ECX
                JE      @Found          {If it is a Complete Match}

{---- Point DI to Next Entry, Check for End-of-Table ---- }
@Next:          Pop     EDI             {Not found: Try Next Entry}
                Add     EDI,EBX
                Cmp     EDI,EDX         {Past end of Strng?}
                JB      @Compare        {No: Keep looking}

@NotFound:      Xor     EAX,EAX         {Yes: Not Found, Set Rtn=0}
                Jmp     @Exit

{---- Restore Registers and Return  ----}
@Found:         Pop     EDI
@Exit:          Pop     EBX             {Restore EBX,EDI,ESI}
                Pop     EDI
                Pop     ESI
  end{Lookup};


{------------------------------------------------------------------------}
{  SCAN a String (Forward or Backward) for the next occurance of any     }
{  one of a group of break characters.                                   }
{    Strng -  String to be scanned (forward or backward).                }
{    Brk   -  List of break characters to scan for. If <Null> then       }
{             the previous list will be used (this saves setup time).    }
{    StPosn - Scan Starting Position in Strng, Posn<0 means Scan         }
{             backwards from Abs(Posn).  Set -Posn at or beyond the Len  }
{             of Strng to scan backward from the end (e.g. Posn=-9999).  }
{    Which -  Returns rel no. (in Brk) of the break char found.          }
{      Rtns: Position of the break char found in Strng, 0 if not found.  }
{   Example: p := Scan('What, Me? No!', ',?!', 6, wh);  Rtns: p=9, wh=2  }
{------------------------------------------------------------------------}
Function Scan(const Strng,Brk: String; StPosn: Integer; var Which:Integer):
         Integer;   Assembler;
  var  StartPos: Integer;
  asm           Push    ESI             {Save ESI,EDI,EBX}
                Push    EDI
                Push    EBX
{---- Setup the Translate Table ----}
                Mov     ESI,EDX         {ESI = @Brk}
                Or      EDX,EDX
                JZ      @GetStr         {If <nil>, Use Prev Break}
                Mov     EDX,[EDX-4]     {Len of <Brk>}
                Or      EDX,EDX         {If <Null>, Use Prev Break}
                JZ      @GetStr
                Push    EAX             {Save Parms: @Strng,<StPosn>}
                Push    ECX

                LEA     EDI,XTbl        {--Clear the 'break' char table--}
                Mov     EBX,EDI         {Save the table addr in EBX}
                Mov     ECX,64          {64 double words}
                Xor     EAX,EAX         { of 00000000h}
                Rep     StoSD

                Mov     ECX,EDX         {--Setup for ECX Brk chars--}
                Xor     EDX,EDX         {Preset char no. cntr}
@SetChar:       LodSB                   {Get next break char}
                Mov     EDI,EAX         {Make it the index into Table}
                Add     EDI,EBX
                Inc     EDX             {Incr char no. cntr}
                Mov     AL,DL
                StoSB                   {Save the char no. in Table}
                Loop    @SetChar
                Pop     ECX             {Restore Parms}
                Pop     EAX

{---- Get Addr and Len of Strng, Adjust if StPosn>0, Set Scan Direction ----}
@GetStr:        Mov     EBX,ECX         {EBX = <StPosn>}
                Mov     StartPos,EBX    {Save Start Position}
                Mov     ESI,EAX         {ESI = @Strng}
                Or      EAX,EAX
                JZ      @NotFound       {If <nil> set Not Found}
                Mov     EAX,[EAX-4]     {Len of <Strng>}
                Mov     EDX,EAX         {Save Len of <Strng>}
                Mov     ECX,EAX         {Check Len of <Strng>}
                JCXZ    @NotFound       {Set Not Found if <Strng> is <Null>}

@AdjBkwd:       Cmp     EBX,0           {--Check <StPosn> for bkwd scan--}
                JE      @NotFound       {Set Not Found if StPosn=0}
                JG      @AdjFwd         {Go Adj for Fwd Scan if StPosn>0}
                STD                     {Set for backward Scan}
                Neg     EBX             {Else Make the Start Posn +}
                Cmp     EBX,ECX         {Check if StPosn>Len(Strng)}
                JLE     @PosnOK         {If not, OK}
                Mov     EBX,ECX         {Else use Len(Strng) as start StPosn}
@PosnOK:        Mov     ECX,EBX         {Use <StPosn> as Scan Len}
                Jmp     @AdjStart

@AdjFwd:        Cmp     EBX,ECX         {Check if StPosn>Len(Strng)}
                JG      @NotFound       {If it is, Set Not Found}
                Sub     ECX,EBX         {Adj Scan Len = Len(Strng)-StPosn+1}
                Inc     ECX
@AdjStart:      Add     ESI,EBX         {Adj Starting Scan Addr}
                Dec     ESI             { = @Strng+StPosn-1}


{---- Scan Strng for Brk Using "Xlat" Instruction ----}
@Xtbl:          LEA     EBX,XTbl        {Tbl Adr for 'Xlat'}
                Xor     EAX,EAX         {Clear high bits of EAX}
@ScanIt:        LodSB                   {Get next char of Strng}
                Xlat                    {Look up in 'break' char table}
                Cmp     AL,0            {Non-zero means its a break char}
                JNE     @Found
                Loop    @ScanIt

@NotFound:      Xor     EAX,EAX         {Set Found Posn to 0}
                Jmp     @Exit

{---- Set Position and Which Break Char Found ----}
@Found:         Mov     EDI,Which       {EDI = @Which}
                StoSD                   {Set no. of break char found}
                Mov     EAX,ECX         {ECX is Found Posn if Bkwrds}
                Cmp     StartPos,0      {Check direction}
                JLE     @Exit           {Done If Bkwrds}
                Mov     EAX,EDX         {Else Calc Indx of char in Strng}
                Sub     EAX,ECX
                Inc     EAX

{---- Restore Registers and Return  ----}
@Exit:          CLD                     {Clear Direction Flag}
                Pop     EBX             {Restore EBX,EDI,ESI}
                Pop     EDI
                Pop     ESI
  end{Scan};


{------------------------------------------------------------------------}
{  SCANCHAR Returns the position of the first/last occurance of a Char   }
{   in Strng starting from Posn. Faster than SCAN if just one char to    }
{   match. And POS can't set a starting position or scan backwards.      }
{    Strng  - String to be scanned (forward or backward).                }
{    OneCh  - The single character to find.                              }
{    StPosn - Scan Starting Position in Strng, Posn<0 means Scan         }
{             backwards from Abs(Posn).  Set -Posn at or beyond the Len  }
{             of Strng to scan backward from the end (e.g. Posn=-9999).  }
{      Rtns: Position of Char if found in Strng, 0 if not found.         }
{  Example:  p := ScanChar(' Hello There', ' ', -99);  Rtns p=7          }
{------------------------------------------------------------------------}
Function ScanChar(const Strng:String; OneCh:Char; StPosn:Integer): Integer;
  Assembler;
  asm           Push    ESI             {Save ESI,EDI,EBX}
                Push    EDI
                Push    EBX
{---- Get Addr and Len of Strng, Adjust if StPosn > 0 ----}
                Mov     EDI,EAX         {EDI = @Strng}
                Or      EAX,EAX
                JZ      @NotFound       {Not Found if <Strng> is <nil>}
                Mov     EBX,ECX         {Value of <StPosn>}
                Mov     ECX,[EAX-4]     {Len of <Strng>}
                JCXZ    @NotFound       {Set Not Found if <Strng> is <Null>}
                Xor     EAX,EAX
                Mov     AL,DL           {<OneCh> to find in AL}
                Mov     EDX,ECX         {Save Len of Strng in EDX}
                Cmp     EBX,0
                JE      @ScanFwd        {If StPosn=0, Use Full Strng }
                JL      @Backward       {If StPosn<0, Scan Backwards}

{---- Setup Start Position and Length for Forward Scan ----}
                Cmp     EBX,ECX         {Compare StPosn to Len(Strng)}
                JG      @NotFound       {If StPosn>Len(Strng), Its Not Found}
                Dec     EBX             {Make Posn Rel Offset in Strng}
                Add     EDI,EBX         {Adjust Start Addr}
                Sub     ECX,EBX         {Adjust Scan Len}

{---- Forward Scan Strng for Char Using ScaSB Instruction ----}
@ScanFwd:       RepNZ   ScaSB           {Scan Strng for the char}
                JNZ     @NotFound
                Mov     EAX,EDX         {Calc Found Posn of char in Strng}
                Sub     EAX,ECX         {= Len(Strng)-ECX}
                Jmp     @Exit

{---- Setup Start Position and Length for Backward Scan ----}
@Backward:      Neg     EBX
                Cmp     EBX,ECX
                JLE     @BkwdOK         {If StPosn>Len(Strng),}
                Mov     EBX,ECX         { Use Len(String) = Last Char}
@BkwdOK:        Dec     BX              {Rel Offset to Start}
                Add     EDI,EBX         {Adjust Start Addr}
                Mov     CX,BX           {Else If Bkwd, Length is inverse}
                Inc     CX

{---- Backward Scan Strng for Char Using ScaSB Instruction ----}
                STD                     {For Backward Scan}
@ScanBkwd:      RepNZ   ScaSB           {Scan Strng for the char}
                JNZ     @NotFound
                Mov     EAX,ECX         {Calc Found Posn of char in Strng}
                Inc     EAX             {= ECX+1}
                Jmp     @Exit

{---- Restore Registers and Return  ----}
@NotFound:      Xor     EAX,EAX         {If Not Found, Set Result to 0}
@Exit:          CLD                     {Clear Direction Flag}
                Pop     EBX             {Restore EBX,EDI,ESI}
                Pop     EDI
                Pop     ESI
  end{ScanChar};


(*THIS PROGRAM UNIT IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED. INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO
QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE UNIT PROVE
DEFECTIVE, YOU ASSUME THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR
CORRECTION. THE AUTHOR IS NOT LIABLE FOR ANY DAMAGE INCLUDING BUT NOT
LIMITED TO: DAMAGE TO HARDWARE, LOSS OF DATA, LOSS OF TIME, OR OUT-DATED
NECKTIES.*)
END.
