{
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
   eXtreme Class Library
   of classes to replace
   Delphi VCL. Non-visual object oriented Pascal programming
   for Windows. XCL allow creating of very small applets
   which therefore can look like usual Windows applications.

   Idea: Kladov Vladimir (C) 1999, bonanzas@online.sinor.ru
   Home: http://members.xoom.com/mr_bonanzas
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
   XCL Wizard tool by Vladimir Kladov (C) 1999
   intended to provide XCL with its own visual
   programming environment. It is including
   following parts:
   - XCL Expert, integrated into IDE and providing
                 XCL Designer with notifications about
                 IDE events and executing its requests
   - XCL Designer, interacting with XCL Expert part
                 and containing:
                 - XCL Forms Visual Editor
                 - XCL Property Inspector
                 - XCL Classes Palette
   - XCL Library (DLL), containing all XCL classes
                 compiled and defined in special format
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
   Here is XCL Library unit, which contains definitions
   for XCL classes (and controls) in form of constant
   arrays. Changing this unit and recompiling one it is
   possible to register new classes onto XCL Palette and
   use it at design-time (without altering XDesigner.exe
   and XCLWizard.bpl/dpl parts of the XCL Wizard).
   
--- ChAr: 13.2.2000
Discovered bug in function EnumAllFonts causing system error in Kernel32
 under W98SE .
 See EnumAllFonts later in this unit.     
}

unit XLib;

interface

uses ShareMem, XCLClsLib,
  windows, messages, XClasses, XVisuals, XWindows, XForms, XApplets,
  XGraphics, XCanvases, XImages, XRotateFonts,
  XControls, XMFCControls, XCalendars, XGrids, XHystogramms,
  XImgButtons, XMenus, XTimers, XSplitPanels, XEdits, XEditClpbrd,
  XEditMouseSel, XEditUndoExt, XFocusDrawing, XCarets,
  XMFCAntiFlick, XResize, XAligns, XKeyboardEvents, XMouseEvents,
  XStreams, XStrLists, XLists, XListEditExt, XListKeySel, XListMouseSel,
  XStrUtils, XDsgnControls;
  //, XBtnRepeats;

const
  XLibVersion : Word = $0093; // Version 0.91 of XCLCLSD4.dll (this)
  NeedDsgnVer : Word = $0093; // need v0.91 of XCL Designer

function CreateForm( Visible : Boolean; Caption : PChar; Rect : PRect; ClsStyle, Style, ExStyle : DWORD;
                     Flags : DWORD; CloseEvnt : XOnEventAccept;
                     ActivateEvnt, SelectEvnt, DeselectEvnt, ResizeEvnt, MoveEvnt : XOnEvent;
                     ClickEvent : XOnDsgnClick; KeyDownEvent, KeyUpEvent : XOnDsgnKey )
                     : TObject;
function CreateFormWindow( Form : TObject ) : HWnd;
function CreateNonvisual( FormObj : TObject; Visible : Boolean; ImgList : THandle; ImgIdx : Integer;
                     Pos : TPoint ) : TObject;
function GetNonvisualWindow( NonVisual : TObject ) : HWnd;
procedure SetNonVisualIcon( NonVisual : TObject; ImgList : THandle;
                            ImgIdx : Integer );

function CheckDesignerVersion( DesignerVersion : Word ) : Boolean;
function GetXLibVersion : Word;

function InitInterface( MainInst : HInst; MainApplet : XApplet ) : Pointer;
procedure UnInstall;
function GetFormReservedNames : PStringArray;
function GetClassesCount: Integer;
function GetClassInfo(Idx: Integer): PXClassInfo;
function GetCreatorString( AObject : TObject; ClsIdx: Integer ) : string;
function GetPropertyValue(AObject: TObject; ClassIdx, PropIdx: Integer): DWORD;
procedure SetPropertyValue(AObject: TObject; ClassIdx, PropIdx: Integer; Value: DWORD);
function NewClassObject( ClsIdx: Integer; Parent : TObject ) : TObject;
procedure SetFormObjectPos( Obj : TObject; Pos : TPoint );
procedure SelectObject( Form : TObject; Obj : TObject );
procedure DeselectObject( Form : TObject; Obj : TObject );

function Prop_ColorAsString( Value : DWORD ) : String;
function Prop_String2Color( const StrValue : string ) : DWORD;
function Prop_BevelEdgesAsDWORD( Value : XBevelEdges ) : DWORD;
function Prop_BevelEdgesFrDWORD( Value : DWORD ) : XBevelEdges;
function Prop_FontStyleAsDWORD( Value : XFontStyle ) : DWORD;
function Prop_FontStyleFrDWORD( Value : DWORD ) : XFontStyle;
function GetFontsCount : Integer;
function GetFontNames( Idx : Integer ) : String;

implementation

var MainInstance : HInst;
    MainAppl : XApplet;

{$R XCLIcons.res}

function Parent_Self( Sender : TObject ) : TObject;
begin
  Result := Sender;
end;

  // Properties arrays for certain classes. Are used in class definitions
  // (follow these arrays). Following fields can be defined to describe
  // a property:
  //    Nam     - name of the property
  //        if name contains dots ('.'), it is presenting field of record,
  //        or property of property object (e.g. Font.Name) and at start
  //        all such properties are collapsed into one line in Property
  //        Inspector.
  //    Typ     - string type definition of property.
  //        - For sets, enums and flags (or for numbers, which could be
  //        partially presented as named constants - like XColor), looks
  //        like list of values in parenthesis (round or square - for sets
  //        and flags last).
  //        - For ordinal types and strings, contains string (like 'Integer').
  //        - For events, contains detailed definition of event procedure
  //        (or function -?) type. If definition is finishing with 'override',
  //        event is creating as [protected] method, overriding inherited one
  //        (e.g., for pseudo-events OnCreateForm/OnDestroyForm procedures
  //        CreateForm/DestroyForm are overriding. Only for forms.)
  //        It is required that first two letters of the event be 'On'.
  //    Eds     - edit styles. Used when property is editing in Property
  //              Inspector. Following styles are available:
  //        esEdit  - typing value with keyboard;
  //        esCombo - dropping down list of available (or most common) values.
  //              If property is not enum, all most common values therefore
  //              are listed in Typ field (above), finishing the list with
  //              special format string (e.g. - $# - to represent value as
  //              hexadecimal, ## - as decimal).
  //        esChkCombo - expanding properties tree into list of checkboxes.
  //              Exclusive with others. Usually corresponding to sets.
  //        esIncDec - DropButtons are shown to increase/decrease property
  //              value in Property Inspector;
  //        esEvent - special editing style for events only.
  //        esCustom - use editing custom procedure, provided with field
  //              Editor (below). Compatible with others.
  //    Editor  - editor procedure for custom editing. Optional.
  //    AsStr   - function to translate property value to string (used in both
  //            Property Inspector to represent property values and when code
  //            is generating to change properties in CreateCustomObjects method
  //            of the form). Can be nil - standard represantation is used
  //            (available for Integer, Boolean, DWord, String, as well as for
  //            sets, enums and flags.
  //    SetStr  - function to translate string to property value. If available,
  //            can be used in Property Inspector to allow editing property
  //            value manually.
  //    DesignProtect - True, if changing of property at design time can not be
  //            performed. In that case, form designer uses Dflt value as value
  //            at start of creating class and performs changing of the value,
  //            storing it internally.
  //    Dflt    - Value by default (immediately after creating the object). It
  //            is ignored if DesignProtect is False and real value of property
  //            is used as its default immediately after creating (but only if
  //            there are no such default among defaults list of the class -
  //            - see below).
type
  XClass_Props = ( _Tag, _end_XClass );

const
  Props_XClass : array[ XClass_Props ] of TXPropInfo =
  (
    //(Nam:'Parent'; Typ:'XClass'; Eds:esComboEdit),
    //(Nam:'ChildCount'; Typ:'Integer'; Eds:[]),
    (Nam:'Tag'; Typ:'Pointer'; Eds:[esEdit]),
    ()
  );

type
  XVisual_Props = ( _Enabled, _Visible, _Left, _Top, _Width, _Height, _end_XVisual );

const
  Props_XVisual: array[ XVisual_Props ] of TXPropInfo =
  (
    (Nam:'Enabled'; Typ:'Boolean'; Eds:esBoolEdit),
    (Nam:'Visible'; Typ:'Boolean'; Eds:esBoolEdit),
    (Nam:'Left'; Typ:'Integer'; Eds:esNumEdit+[esConfirm]),
    (Nam:'Top'; Typ:'Integer'; Eds:esNumEdit+[esConfirm]),
    (Nam:'Width'; Typ:'Integer'; Eds:esNumEdit+[esConfirm]),
    (Nam:'Height'; Typ:'Integer'; Eds:esNumEdit+[esConfirm]),
    ()
  );

const
  Values_XColor = '(clScrollBar,clBackground,clActiveCaption,clInactiveCaption,'+
    'clMenu,clWindow,clWindowFrame,clMenuText,clWindowText,clCaptionText,'+
    'clActiveBorder,clInactiveBorder,clAppWorkSpace,clHighlight,clHighlightText,'+
    'clBtnFace,clBtnShadow,clGrayText,clBtnText,clInactiveCaptionText,clBtnHighlight,'+
    'cl3DDkShadow,cl3DLight,clInfoText,clInfoBk,clBlack,clMaroon,clGreen,clOlive,'+
    'clNavy,clPurple,clTeal,clGray,clSilver,clRed,clLime,clYellow,clBlue,clFuchsia,'+
    'clAqua,clLtGray,clDkGray,clWhite,clNone,clDefault,$#)';

const
  Flags_ClsStyle : array[ 0..10 ] of DWORD = (
         CS_BYTEALIGNCLIENT,CS_BYTEALIGNWINDOW,CS_CLASSDC,CS_DBLCLKS,
         CS_GLOBALCLASS,CS_HREDRAW,CS_NOCLOSE,CS_OWNDC,CS_PARENTDC,
         CS_SAVEBITS,CS_VREDRAW);

  Flags_Style : array[ 0..21 ] of DWORD = (
         WS_BORDER,WS_CAPTION,WS_CHILD,WS_CLIPCHILDREN,WS_CLIPSIBLINGS,
         WS_DISABLED,WS_DLGFRAME,WS_GROUP,WS_HSCROLL,WS_ICONIC,WS_MAXIMIZE,
         WS_MAXIMIZEBOX,WS_MINIMIZEBOX,WS_OVERLAPPED,WS_POPUP,WS_SIZEBOX,
         WS_SYSMENU,WS_TABSTOP,WS_THICKFRAME,WS_TILED,WS_VISIBLE,WS_VSCROLL);

  Flags_ExStyle : array[ 0..18 ] of DWORD = (
         WS_EX_ACCEPTFILES,WS_EX_APPWINDOW,WS_EX_CLIENTEDGE,
         WS_EX_CONTEXTHELP,WS_EX_CONTROLPARENT,WS_EX_DLGMODALFRAME,WS_EX_LEFT,
         WS_EX_LEFTSCROLLBAR,WS_EX_LTRREADING,WS_EX_MDICHILD,WS_EX_NOPARENTNOTIFY,
         WS_EX_RIGHT,WS_EX_RIGHTSCROLLBAR,WS_EX_RTLREADING,WS_EX_STATICEDGE,
         WS_EX_TOOLWINDOW,WS_EX_TOPMOST,WS_EX_TRANSPARENT,WS_EX_WINDOWEDGE);

type
  XWindow_Props = ( _ClsStyle, _Style, _ExStyle, _Caption, _Color,
                    _WindowState, _OnClose, _end_XWindow );

const
  Props_XWindow: array[ XWindow_Props ] of TXPropInfo =
  (
    (Nam:'ClsStyle'; Typ:'[CS_BYTEALIGNCLIENT,CS_BYTEALIGNWINDOW,CS_CLASSDC,CS_DBLCLKS,'+
         'CS_GLOBALCLASS,CS_HREDRAW,CS_NOCLOSE,CS_OWNDC,CS_PARENTDC,CS_SAVEBITS,CS_VREDRAW]';
         Flg:@Flags_ClsStyle; Eds: [ esEdit, esChkCombo, esConfirm ]; DesignProtect:True),
    (Nam:'Style'; Typ:'[WS_BORDER,WS_CAPTION,WS_CHILD,WS_CLIPCHILDREN,WS_CLIPSIBLINGS,'+
             'WS_DISABLED,WS_DLGFRAME,WS_GROUP,WS_HSCROLL,WS_ICONIC,WS_MAXIMIZE,'+
             'WS_MAXIMIZEBOX,WS_MINIMIZEBOX,WS_OVERLAPPED,WS_POPUP,WS_SIZEBOX,'+
             'WS_SYSMENU,WS_TABSTOP,WS_THICKFRAME,WS_TILED,WS_VISIBLE,WS_VSCROLL]';
             Flg:@Flags_Style; Eds: [ esEdit, esChkCombo, esConfirm ]; DesignProtect:True),
    (Nam:'ExStyle'; Typ:'[WS_EX_ACCEPTFILES,WS_EX_APPWINDOW,WS_EX_CLIENTEDGE,'+
             'WS_EX_CONTEXTHELP,WS_EX_CONTROLPARENT,WS_EX_DLGMODALFRAME,WS_EX_LEFT,'+
             'WS_EX_LEFTSCROLLBAR,WS_EX_LTRREADING,WS_EX_MDICHILD,WS_EX_NOPARENTNOTIFY,'+
             'WS_EX_RIGHT,WS_EX_RIGHTSCROLLBAR,WS_EX_RTLREADING,WS_EX_STATICEDGE,'+
             'WS_EX_TOOLWINDOW,WS_EX_TOPMOST,WS_EX_TRANSPARENT,WS_EX_WINDOWEDGE]';
             Flg:@Flags_ExStyle; Eds: [ esEdit, esChkCombo, esConfirm ]; DesignProtect:True),
    (Nam:'Caption'; Typ:'String'; Eds:[esEdit]),
    (Nam:'Color'; Typ:Values_XColor; Eds:esComboEdit;
                  AsStr:Prop_ColorAsString; SetStr:Prop_String2Color ),
    (Nam:'WindowState'; Typ:'(xwsNormal,xwsMinimized,xwsMaximized)'; eds:esCase;
             DesignProtect:True ),
    (Nam:'OnClose'; Typ:'procedure (Sender: TObject, var Accept:Boolean); of object';
              Eds:[esEvent]),
    //(Nam:'Handle'; Typ:'Integer'; eds:[]),
    ()
  );

  Dflts_XWindow : array[ 0..4 ] of PChar = ( 'Style = WS_VISIBLE or WS_CLIPCHILDREN or '+
                'WS_CLIPSIBLINGS or WS_CAPTION or WS_SYSMENU or WS_MINIMIZEBOX or '+
                'WS_MAXIMIZEBOX or WS_BORDER or WS_THICKFRAME',
                'ClsStyle = CS_OWNDC', 'ExStyle = WS_EX_CONTROLPARENT',
                'Color = clBtnFace', nil );

type
  XForm_Props = ( _AutoAlign, _Margin, _On_Message, _OnKeyDown, _OnKeyUp, _OnKeyPress,
                  _OnMouseDown, _OnMouseMove, _OnMouseUp, _end_XForm );

const
  Props_XForm : array[ XForm_Props ] of TXPropInfo =
  (
    (Nam:'AutoAlign'; Typ:'Boolean'; Eds:esBoolEdit),
    (Nam:'Margin'; Typ:'Integer'; Eds:esNumEdit+[esConfirm]),
    (Nam:'On_Message'; Typ:'procedure (Sender: TObject; var Msg: TMsg; '+
               'var Rslt:Integer; var Handled:Boolean); of object'; Eds:[esEvent]),
    (Nam:'OnKeyDown'; Typ:'procedure (Sender:TObject; var Key:Word;'+
               ' Shift:XShifState); of object'; Eds:[esEvent] ),
    (Nam:'OnKeyUp'; Typ:'procedure (Sender:TObject; var Key:Word;'+
               ' Shift:XShiftState); of object'; Eds:[esEvent] ),
    (Nam:'OnKeyPress'; Typ:'procedure (Sender:TObject; var Key:Char); of object';
               Eds:[esEvent] ),
    (Nam:'OnMouseDown'; Typ:'procedure (Sender:TObject; Button: XMouseButton;'+
               ' Shift : XShiftState; X, Y: Integer); of object'; Eds:[esEvent]),
    (Nam:'OnMouseMove'; Typ:'procedure (Sender:TObject;'+
               ' Shift:XShiftState; X,Y:Integer); of object'; Eds:[esEvent]),
    (Nam:'OnMouseUp'; Typ:'procedure (Sender:TObject; Button: XMouseButton;'+
               ' Shift:XShiftState; X,Y:Integer); of object'; Eds:[esEvent]),
    ()
  );

type
  XCustomForm_Props = ( _Visible_XCustomForm, _OnCreateForm, _OnDestroyForm, _end_XCustomForm );

const
  Props_XCustomForm : array[ XCustomForm_Props ] of TXPropInfo =
  (
    (Nam:'Visible'; Typ:'Boolean'; Eds:esBoolEdit; DesignProtect:True),
    (Nam:'<OnCreateForm>'; Typ:'procedure; override;'; Eds: [esEvent]),
    (Nam:'<OnDestroyForm>'; Typ:'procedure; override;'; Eds: [esEvent]),
    ()
  );

  Dflts_XCustomForm : array[ 0..1 ] of PChar = ( 'Visible = True', nil );

type
  XApplet_Props = ( _OleUsed, _end_XApplet );

const
  Props_XApplet : array[ XApplet_Props ] of TXPropInfo =
  (
    (Nam:'OleUsed'; Typ:'Boolean'; Eds:esBoolEdit; DesignProtect:True ),
    ()
  );

type
  XControl_Props = ( _Align, _OnKeyDown_XControl, _OnKeyUp_XControl, _OnKeyPress_XControl,
                     _OnMouseDown_XControl, _OnMouseMove_XControl, _OnMouseUp_XControl, _end_XControl );

const
  Props_XControl : array[ XControl_Props ] of TXPropInfo =
  (
    (Nam:'Align'; Typ:'(alNone,alLeft,alTop,alRight,alBottom,alClient)'; Eds:esCase ),
    //(Nam:'TabOrder'; Typ:'Integer'; Eds:[]), // ReadOnly
    (Nam:'OnKeyDown'; Typ:'procedure (Sender:TObject; var Key:Word;'+
               ' Shift:XShiftState); of object'; Eds:[esEvent] ),
    (Nam:'OnKeyUp'; Typ:'procedure (Sender:TObject; var Key:Word;'+
               ' Shift:XShiftState); of object'; Eds:[esEvent] ),
    (Nam:'OnKeyPress'; Typ:'procedure (Sender:TObject; var Key:Char); of object';
               Eds:[esEvent] ),
    (Nam:'OnMouseDown'; Typ:'procedure (Sender:TObject; Button: XMouseButton;'+
               ' Shift : XShiftState; X, Y: Integer); of object'; Eds:[esEvent]),
    (Nam:'OnMouseMove'; Typ:'procedure (Sender:TObject;'+
               ' Shift:XShiftState; X,Y:Integer); of object'; Eds:[esEvent]),
    (Nam:'OnMouseUp'; Typ:'procedure (Sender:TObject; Button: XMouseButton;'+
               ' Shift:XShiftState; X,Y:Integer); of object'; Eds:[esEvent]),
    ()
  );

type
  XCustomControl_Props = ( _OnChangeAppearance, _end_XCustomControl );

const
  Props_XCustomControl: array[ XCustomControl_Props ] of TXPropInfo =
  (
    (Nam:'OnChangeAppearance'; Typ:'procedure (Sender:TObject); of object'; Eds:[esEvent]),
    ()
  );

type
  XLabel_Props = ( _Alignment, _AutoSize_XLabel, _Caption_XLabel,
                   _Font_Name_XLabel, _Font_Style_XLabel, _Font_Height_XLabel,
                   _Font_Color_XLabel, _Font_Orientation_XLabel, _Font_Pitch_XLabel,
                   _Font_Charset_XLabel, _Color_XLabel, _VAlign, _end_XLabel );

const
  Props_XLabel: array[ XLabel_Props ] of TXPropInfo =
  (
    (Nam:'Alignment'; Typ:'(taLeft,taCenter,taRight)'; Eds:esCase),
    (Nam:'AutoSize'; Typ:'Boolean'; Eds:esBoolEdit),
    (Nam:'Caption'; Typ:'String'; Eds:[esEdit]),
    (Nam:'Font.Name'; Typ:'String'; Eds:esComboEdit+[esConfirm]; GetValCount: GetFontsCount; GetVal: GetFontNames),
    (Nam:'Font.Style'; Typ:'[fsBold,fsItalic,fsUderline,fsStrikeOut]'; Eds:[esChkCombo]),
    (Nam:'Font.Height'; Typ:'Integer'; Eds:esNumEdit+[esConfirm]),
    (Nam:'Font.Color'; Typ:Values_XColor; Eds:esComboEdit;
                  AsStr:Prop_ColorAsString; SetStr:Prop_String2Color),
    (Nam:'Font.Orientation'; Typ:'Integer'; Eds:[esEdit,esConfirm]),
    (Nam:'Font.Pitch'; Typ:'(fpDefault,fpVariable,fpFixed)'; Eds: esCase),
    (Nam:'Font.Charset'; Typ:'0..255'; Eds:esNumEdit),
    (Nam:'Color'; Typ:Values_XColor; Eds:esComboEdit;
                  AsStr:Prop_ColorAsString; SetStr:Prop_String2Color),
    (Nam:'VAlign'; Typ:'(vaTop,vaCenter,vaBottom)'; Eds:esCase),
    ()
  );

procedure CreateProc_XLabel( Self_ : TObject; Name : PChar );
begin
  (Self_ as XLabel).Caption := Name;
end;

type
  XBevel_Props = ( _BevelEdges, _BevelInner, _BevelOuter, _BevelWidth,
                   _BorderWidth, _Color_XBevel, _end_XBevel );

const
  Props_XBevel: array[ XBevel_Props ] of TXPropInfo =
  (
    (Nam:'BevelEdges'; Typ:'[beLeft,beTop,beRight,beBottom]'; Eds:[esChkCombo]),
    (Nam:'BevelInner'; Typ:'(bvNone,bvRaised,bvLowered)'; Eds:esCase),
    (Nam:'BevelOuter'; Typ:'(bvNone,bvRaised,bvLowered)'; Eds:esCase),
    (Nam:'BevelWidth'; Typ:'Integer'; Eds:esNumEdit),
    (Nam:'BorderWidth'; Typ:'Integer'; Eds:esNumEdit),
    (Nam:'Color'; Typ:Values_XColor;Eds:esComboEdit;
                  AsStr:Prop_ColorAsString; SetStr:Prop_String2Color),
    ()
  );

type
  XPaint_Props = ( _OnPaint, _Color_XPaint, _end_XPaint );

const
  Props_XPaint: array[ XPaint_Props ] of TXPropInfo =
  (
    (Nam:'OnPaint'; Typ:'procedure (Sender: TObject; Canvas: XCanvas; Rect: TRect); of object';
     Eds:[esEvent]),
    (Nam:'Color'; Typ:Values_XColor; Eds: esComboEdit;
                  AsStr:Prop_ColorAsString; SetStr:Prop_String2Color),
    ()
  );

type
  NonVisual_Props = ( _Left_NV, _Top_NV, _end_NV );

const
  Props_NonVisual: array[ NonVisual_Props ] of TXPropInfo =
  (
    (Nam:'*Left'; Typ:'Integer'; Eds:[]; DesignProtect: False),
    (Nam:'*Top'; Typ:'Integer'; Eds:[]; DesignProtect: False),
    ()
  );

type
  ZTimer_Props = ( _Interval, _Enabled_ZTimer, _OnTimer, _end_ZTimer );

const
  Dflts_ZTimer : array[ 0..2 ] of PChar = ( 'Interval = 1000', 'Enabled = False', nil );

  Props_ZTimer: array[ ZTimer_Props ] of TXPropInfo =
  (
    (Nam:'Interval'; Typ:'Integer'; Eds: esNumEdit + [esConfirm]; DesignProtect: True),
    (Nam:'Enabled'; Typ:'Boolean'; Eds: esBoolEdit; DesignProtect: True; SetAfter: True ),
    (Nam:'OnTimer'; Typ:'procedure (Sender: TObject); of object'; Eds:[esEvent]),
    ()
  );

  // Arrays of addon definitions for certain classes to generate calls
  // of add-ons automatically. Add-on definition consists of following
  // field:
  //    ACall    - call string, which will be included into dpr before
  //               creating of the Applet. If empty, the definition can
  //               be used to include some of needed unit into 'uses'
  //               (in last case Usage=auMust usually and the add-on
  //               will be used silently - without listing it in a list
  //               of collected ad-ons. Also such add-ons will affect
  //               not only dpr-unit 'uses' clause, but 'uses' clause for
  //               all units, containing references to given class or
  //               its descendants).
  //    AUnit    - unit name (without .pas extention) - to list it among
  //               other ones in 'uses' clause of the dpr-file (to make
  //               possible compiling it after inserting call above).
  //    Usage    - usage of the add-on for given class:
  //          auPossible (default) - not used by default, but add-on
  //                     can be checked in the list of all collected
  //                     add-ons by the developer to make it used;
  //          auDesired  - used by default, but can be unchecked to
  //                     stop of using it;
  //          auMust      - used by default, and can not be unchecked
  //                     (only if class can not work without the add-on).
  //                     See also comment to ACall:'' above.
  //          auNever     - not used and is disabled to use with given
  //                      control or class.
  //    Descr    - Brief description string. To use as small tooltip
  //               help in add-ons checklist. Not necessary.
const
  Addons_XForm : array[ 0..9 ] of TXAddonInfo =
  (
    ( ACall: ''; AUnit: 'XClasses'; Usage: auMust ),
    ( ACall: ''; AUnit: 'XVisuals'; Usage: auMust ),
    ( ACall: ''; AUnit: 'XWindows'; Usage: auMust ),
    ( ACall: 'UseAligner'; AUnit: 'XAligns'; Usage: auDesired;
      Descr: 'Allows to align controls on form using its property Align.' ),
    ( ACall: 'UseAutoSizer'; AUnit: 'XFormAutoSize'; Usage: auNever;
      Descr: 'Developed to use in manually created XCL projects to '+
      'simplify placing of controls and resizing its parents automatically. '+
      'Must not be used in XCL projects, created with XCL Wizard (visually).' ),
    ( ACall: 'UseKeyboard'; AUnit: 'XKeyboardEvents'; Usage: auDesired;
      Descr: 'Privides switching between controls on form, which can be '+
      'focused (buttons, edit controls and so on).' ),
    ( ACall: 'UseStdMouse'; AUnit: 'XMouseEvents'; Usage: auPossible;
      Descr: 'Provides switching between controls by clicking its with mouse.' ),
    ( ACall: 'UseControlAutoPlacer'; AUnit: 'XControlAutoPlace'; Usage: auNever;
      Descr: 'Developed to simplify placing of controls in manually created '+
      'XCL projects. Not needed (and not recommended) for XCL projects, '+
      'created visually (using XCL Wizard).' ),
    ( ACall: 'UseResizeAntiFlicker'; AUnit: 'XResize'; Usage: auPossible;
      Descr: 'Useful for create less flickering resizeable forms with different '+
      'side aligned controls.' ),
    ()
  );

  Addons_XControl : array[ 0..1 ] of TXAddonInfo =
  (
    ( ACall: ''; AUnit: 'XGraphics'; Usage: auMust ),
    ()
  );

  Addons_XCustomControl : array[ 0..4 ] of TXAddonInfo =
  (
    ( ACall: 'UseCanvases'; AUnit: 'XCanvases'; Usage: auMust;
      Descr: 'Since descendants of XCustomControl need Canvas:XCanvas to perform '+
      'painting, this add-on will be added to create canvas objects for every '+
      'creating XWindow descendant - to use it later in DoPaint method.' ),
    ( ACall: 'UseCanvasObjects'; AUnit: 'XCanvasObjects'; Usage: auPossible;
      Descr: 'Can be used to economy GDI usage during executing by the way of '+
      'preventing of duplicating of graphic objects.' ),
    ( ACall: ''; AUnit: 'XGraphics'; Usage: auMust ),
    ( ACall: 'UseFontRotator'; AUnit: 'XRotateFonts'; Usage: auPossible;
      Descr: 'Allows rotated text in XCustomControl descendeants changing its '+
      'Font.Orientation property (e.g. 900 to rotate onto 90 degrees).' ),
    ()
  );


  // Array of classes data. Contains all essential info about classes,
  // compiled with the library:
  //    Page       - default page tab on classes palette
  //    Name       - class name
  //    CTyp       - class type
  //    Kind       - kind of class:
  //        ckForm   - class can be used as form. Not shown on palette.
  //                   Must be the single class of this kind (in current version).
  //        ckControl- class can be used as visual control from palette.
  //        ckChild  - class can be used from palette, but not visual.
  //                   (Similar to TComponent in VCL). Not shown on form,
  //                   and can be accessed from list of non-visuals.
  //                   Always has form as Parent when dropped at design time.
  //        ckObject - class derived from TObject (not from XClass).
  //                   Similar to ckChild, but its destruction is adding
  //                   to DestroyCustomObjects method. Usually name of
  //                   such class is starting from Z (e.g., ZTimer).
  //        ckBase   - class is not available to create instances at
  //                   design time. Intended only to re-use properties in
  //                   descendant classes.
  //    CUnit      - unit name (to automate filling of 'uses' clause).
  //                 If not defined for class, value for its ancestor is used.
  //    Ancestor   - name of ancesting class. Needed to find inherited
  //                 properties and events.
  //    Props      - pointer to properties array of the class, "published"
  //                 to adjust at design time (see properties above).
  //    Parent     - function which is returning target control when new
  //                 one is dropped onto given control at design time.
  //                 Ignored for non-controls.
  //    DsgnActive - function which must return True for certain controls
  //                 which could handle mouse / keyboard events in some
  //                 cases. This allows to some controls to adjust its
  //                 appearance at design time not only by using Property
  //                 Inspector, but visually with mouse or keyboard.
  //                    E.g., it is possible to create a function for
  //                 XSplitPanel, which could handle mouse moving while left
  //                 mouse button is not pressed, and to change cursor over
  //                 splits. And handle mouse down and following dragging of
  //                 splits, if LMB was first pressed at the split.
  //                    It is also possible to handle right click by such
  //                 way to popup menu with special items (e.g. New Tab, New
  //                 Page for XTabs and XTabbedNotebook and so on).
  //    Dflts - array of strings with default values for properties, which
  //            are protected at design time. If correspondent default is not
  //            defined here, it will be read using GetPropertyValue as usual.
  //    Editor - custom editor activation procedure (optional). With this
  //            procedure, it is possible to create some special editing for
  //            certain classes.
  //    CreateProc - procedure, which is called when object is creating on
  //            form FIRST time (when object is dropped from the palette).
  //            E.g., label can set its Caption to be equal to its name.
  //    Addons - array of add-ons, created for work with given class, and of
  //            units, used by the class, to include its in 'uses' clause of
  //            a unit, containing references onto the class and its descendants.
type TXCLClasses = ( cXClass, cXVisual, cXWindow, cXForm, cXCustomForm, cXApplet,
      cXControl, cXCustomControl, cXLabel, cXCustomBevel, cXBevel, cxPaint,
      cNonVisual, cZTimer );

var
  ClassesData : array[ TXCLClasses ] of TXClassInfo =
  (
    ( Page: nil; Name: 'XClass'; CTyp: XClass; Kind: ckBase; CUnit: 'XClasses';
      Ancestor: -1; Props: @Props_XClass ),
    ( Page: nil; Name: 'XVisual'; CTyp: XVisual; Kind: ckBase; CUnit: 'XVisuals';
      Ancestor : Ord(cXClass); Props: @Props_XVisual ),
    ( Page: nil; Name: 'XWindow'; CTyp: XWindow; Kind: ckBase; CUnit: 'XWindows';
      Ancestor : Ord(cxVisual); Props: @Props_XWindow; Dflts: @Dflts_XWindow ),
    ( Page: nil; Name: 'XForm'; CTyp: XForm; Kind: ckBase; CUnit: 'XForms';
      Ancestor : Ord(cXWindow); Props: @Props_XForm; Addons: @Addons_XForm ),
    ( Page: nil; Name: 'XCustomForm'; CTyp: XCustomForm; Kind: ckForm; CUnit: 'XForms';
      Ancestor : Ord(cXForm); Props: @Props_XCustomForm; Parent: Parent_Self;
      Dflts: @Dflts_XCustomForm ),
    ( Page: nil; Name: 'XApplet'; CTyp: XApplet; Kind: ckApplet; CUnit : 'XApplets';
      Ancestor : Ord(cXWindow); Props: @Props_XApplet; Parent: nil ),
    ( Page: nil; Name: 'XControl'; CTyp: XControl; Kind: ckBase; CUnit: 'XForms';
      Ancestor : Ord(cXVisual); Props: @Props_XControl; Addons: @Addons_XControl ),
    ( Page: nil; Name: 'XCustomControl'; CTyp: XCustomControl; Kind: ckBase; CUnit: 'XControls';
      Ancestor: Ord(cXControl); Props: @Props_XCustomControl; Addons: @Addons_XCustomControl ),
    ( Page: 'Standard'; Name: 'XLabel'; CTyp: XLabel; Kind: ckControl; CUnit: 'XControls';
      Ancestor : Ord(cXCustomControl); Props: @Props_XLabel; Parent: nil;
      CreateProc: CreateProc_XLabel ),
    ( Page: nil; Name: 'XCustomBevel'; CTyp: XCustomBevel; Kind: ckBase; CUnit: 'XControls';
      Ancestor : Ord(cXCustomControl); Props: nil),
    ( Page: 'Standard'; Name: 'XBevel'; CTyp: XBevel; Kind: ckControl; CUnit: 'XControls';
      Ancestor : Ord(cXCustomBevel); Props: @Props_XBevel; Parent: Parent_Self ),
    ( Page: 'Standard'; Name: 'XPaint'; CTyp: XPaint; Kind: ckControl; CUnit: 'XControls';
      Ancestor: Ord(cxCustomControl); Props: @Props_XPaint; Parent: nil ),
    ( Page: nil; Name: 'non-visual'; Kind: ckBase; Ancestor: -1;
      Props: @Props_NonVisual ),
    ( Page: 'Non-Visual'; Name: 'ZTimer'; CTyp: ZTimer; Kind: ckObject; CUnit: 'XTimers';
      Ancestor: Ord(cNonVisual); Props: @Props_ZTimer; Dflts: @Dflts_ZTimer )
  );

function StrEq( const S1, S2 : String ) : Boolean;
begin
  Result := (Length( S1 ) = Length( S2 )) and
            (LowerCase( S1 ) = LowerCase( S2 ));
end;

//   NewClassObject
// This function allows to create classes instances at "design" time.
// Parameters:
//   ClassName - name of class which instance is required to be created;
//   Parent - object to become parent of newly created object;
// (When forms are creating in designer, really its Parent must be nil,
// because Applet of XDesigner can be of other time and incompatible
// with newest Applet included into XDesigner itself).
function NewClassObject( ClsIdx: Integer; Parent : TObject ) : TObject;
var Bv : XBevel;
begin
    Result := nil;
    case TXCLClasses( ClsIdx ) of
    cXClass:        ;
    cXVisual:       ;
    cXWindow:       ;
    cxForm:         ;
    cxCustomForm:
       begin
         Result := XCustomForm.Create( Applet );
         (Result as XCustomForm).CreateWindow;
       end;
    cXApplet:       ;
    cXControl:      ;
    cXCustomControl:;
    cXLabel: Result := NewLabel( Parent as XClass, '' );
    cXCustomBevel:  ;
    cXBevel: Result := XBevel.Create( Parent as XClass );
    cXPaint:
      begin
        Result := XBevel.Create( Parent as XClass );
        (Result as XBevel).BevelWidth := 0;
        //(Result as XBevel).BorderWidth := 1;
        Bv := XBevel.Create( Result as XClass );
        Bv.BevelWidth := 0;
        Bv.BorderWidth := 0;
        Bv.Align := alClient;
        Bv.Enabled := False;
        (Result as XBevel).Color := Color2RGB( clBtnFace ) xor $FFFFFF;
      end;
    end;
end;

procedure SetFormObjectPos( Obj : TObject; Pos : TPoint );
var F : XForm;
begin
  if Obj is XControl then
  begin
    if ((Obj as XControl).Parent <> (Obj as XControl).ParentForm)
    and ((Obj as XControl).Parent is XControl) then
    begin
      F := (Obj as XControl).ParentForm;
      if F <> nil then
      begin
        Pos := F.ClientToScreen( Pos );
        Pos := ((Obj as XControl).Parent as XControl).ScreenToClient( Pos );
      end;
    end;
    (Obj as XControl).Position := Pos;
  end
     else
  if Obj is XDsgnNonvisual then
  begin
  // other for nonvisual objects:
    (Obj as XDsgnNonVisual).Position := Pos;
  end;
end;

//   GetCreatorString
// This function returns string which is used to generate creations of
// forms and controls (forms - in dpr and controls - in CreateCustomControls
// method of XCustomForm descendants). Returning string contains only right
// side of assignment and can include macros, such as:
//    %PARENT% - to substitute with parent control name;
//    %PARENTTYPE% - type of the parent (the same as %FORMTYPE% if parent
// is the form itself);
//    %FORM% - global name of form variable;
//    %FORMTYPE% - to substitute with class name of form (must be derived
// from XCustomForm).
//    %propname% - to substitute with a property value of the object
//
// Returning empty string means that designer will generate line:
// %ObjectName% := %ObjectType%.Create( %PARENT% );
// (after substituting of %Macros%-es with appropriate string values).
function GetCreatorString( AObject : TObject; ClsIdx: Integer ) : string;
begin
    Result := '';
    case TXCLClasses( ClsIdx ) of
    cXClass:        ;
    cXVisual:       ;
    cXWindow:       ;
    cXForm:         ;
    cXCustomForm: Result := '%FORMTYPE%.Create( Applet );';
    cXApplet:       ;
    cXControl:      ;
    cXCustomControl:;
    cXLabel:
      begin
        Result := 'NewLabel( %PARENT%, %Caption% );';
        if (AObject as XLabel).AutoSize then
           Result := Result + ' // Width=%Width%; Height=%Height%;'
           // Adding such comment prevents from generating lines
           // for Width and Height
      end;
    cXCustomBevel:  ;
    cXBevel: //Result := 'XBevel.Create( %PARENT% );';
           ;
    cXPaint:        ;
    cZTimer: Result := 'ZTimer.Create;';
    end;
end;

var StrBuf : PChar;
function ReturnString( const Str : String ) : DWORD;
begin
  if StrBuf <> nil then
     FreeMem( StrBuf );
  GetMem( StrBuf, Length( Str ) + 1 );
  Move( Str[ 1 ], StrBuf[ 0 ], Length( Str ) + 1 );
  Result := DWORD( StrBuf );
end;

function GetPropertyValue(AObject: TObject; ClassIdx: Integer; PropIdx: Integer): DWORD;
begin
  Result := 0;
  try
    case TXCLClasses( ClassIdx ) of
    cXClass:
      case XClass_Props( PropIdx ) of
      _Tag: Result := DWord( (AObject as XClass).Tag );
      end;
    cXVisual:
      case XVisual_Props( PropIdx ) of
      _Enabled: Result := DWord( (AObject as XVisual).Enabled );
      _Visible: Result := DWord( (AObject as XVisual).Visible );
      _Left:    Result := (AObject as XVisual).Left;
      _Top:     Result := (AObject as XVisual).Top;
      _Width:   Result := (AObject as XVisual).Width;
      _Height:  Result := (AObject as XVisual).Height;
      end;
    cXWindow:
      case XWindow_Props( PropIdx ) of
      _ClsStyle: Result := (AObject as XWindow).ClsStyle;
      _ExStyle:  Result := (AObject as XWindow).ExStyle;
      _Style:    Result := (AObject as XWindow).Style;
      _Caption:  Result := ReturnString( (AObject as XWindow).Caption );
      _Color:    Result := DWord( (AObject as XWindow).Color );
      _WindowState: Result := DWord( (AObject as XWindow).WindowState );
      end;
    cXForm:
      case XForm_Props( PropIdx ) of
      _AutoAlign: Result := DWord( (AObject as XForm).AutoAlign );
      _Margin:    Result := DWord( (AObject as XForm).Margin );
      end;
    cXCustomForm: ;
    cXApplet:
      case XApplet_Props( PropIdx ) of
      _OleUsed:  Result := DWORD( (AObject as XApplet).OleUsed );
      end;
    cXControl:
      case XControl_Props( PropIdx ) of
      _Align: Result := DWORD( (AObject as XControl).Align );
      end;
    cXCustomControl:;
    cXLabel:
      case XLabel_Props( PropIdx ) of 
      _Alignment:         Result := DWORD( (AObject as XLabel).Alignment );
      _AutoSize_XLabel:   Result := DWORD( (AObject as XLabel).Autosize );
      _Caption_XLabel:    Result := ReturnString( (AObject as XLabel).Caption );
      _Font_Name_XLabel:  Result := ReturnString( (AObject as XLabel).Font.Name );
      _Font_Style_XLabel: Result := Prop_FontStyleAsDWORD( (AObject as XLabel).Font.Style );
      _Font_Height_XLabel:Result := DWORD( (AObject as XLabel).Font.Height );
      _Font_Color_XLabel: Result := DWORD( (AObject as XLabel).Font.Color );
      _Font_Orientation_XLabel: Result := DWORD( (AObject as XLabel).Font.Orientation );
      _Font_Pitch_XLabel: Result := DWORD( (AObject as XLabel).Font.Pitch );
      _Font_Charset_XLabel: Result := DWORD( (AObject as XLabel).Font.Charset );
      _Color_XLabel:      Result := DWORD( (AObject as XLabel).Color );
      _VAlign:            Result := DWORD( (AObject as XLabel).VAlign );
      end;
    cXCustomBevel:;
    cXBevel:
      case XBevel_Props( PropIdx ) of
      _BevelEdges:  Result := Prop_BevelEdgesAsDWORD((AObject as XBevel).BevelEdges );
      _BevelInner:  Result := DWORD( (AObject as XBevel).BevelInner );
      _BevelOuter:  Result := DWORD( (AObject as XBevel).BevelOuter );
      _BevelWidth:  Result := DWORD( (AObject as XBevel).BevelWidth );
      _BorderWidth: Result := DWORD( (AObject as XBevel).BorderWidth );
      _Color_XBevel:Result := DWORD( (AObject as XBevel).Color );
      end;
    cXPaint: // emulated by XBevel
      case XPaint_Props( PropIdx ) of
      _Color_XPaint:Result := DWORD( ((AObject as XBevel).Children[0] as XBevel).Color );
      end;
    cNonVisual: // Pseudo-properties, invisible in Inspector
      case NonVisual_Props( PropIdx ) of
      _Left_NV: Result := (AObject as XDsgnNonvisual).Position.x;
      _Top_NV:  Result := (AObject as XDsgnNonvisual).Position.y;
      end;
    cZTimer: ;// all props are protected and designer maintains it virtually
    end;
  finally
  end;
end;

procedure SetPropertyValue(AObject: TObject; ClassIdx, PropIdx: Integer; Value: DWORD);
var DfltFont : XFont;
begin
  try
    case TXCLClasses( ClassIdx ) of
    cXClass:
      case XClass_Props( PropIdx ) of
      _Tag: (AObject as XClass).Tag := Pointer(Value);
      end;
    cXVisual:
      case XVisual_Props( PropIdx ) of
      _Enabled: (AObject as XVisual).Enabled := Boolean(Value);
      _Visible: (AObject as XVisual).Visible := Boolean(Value);
      _Left:    (AObject as XVisual).Left := Integer(Value);
      _Top:     (AObject as XVisual).Top := Integer(Value);
      _Width:   (AObject as XVisual).Width := Integer(Value);
      _Height:  (AObject as XVisual).Height := Integer(Value);
      end;
    cXWindow:
      case XWindow_Props( PropIdx ) of
      _Caption: (AObject as XWindow).Caption := PChar( Value );
      _Color:   (AObject as XWindow).Color := XColor( Value );
      end;
    cXForm:
      case XForm_Props( PropIdx ) of
      _AutoAlign: (AObject as XForm).AutoAlign := Boolean( Value );
      _Margin:    (AObject as XForm).Margin := Integer(Value);
      end;
    cXCustomForm:;
    cXApplet:;
    cXControl:
      case XControl_Props( PropIdx ) of
      _Align: (AObject as XControl).Align := XAlign( Value );
      end;
    cXCustomControl:;
    cXLabel:
      case XLabel_Props( PropIdx ) of 
      _Alignment:         (AObject as XLabel).Alignment := XTextAlignment( Value );
      _AutoSize_XLabel:   (AObject as XLabel).Autosize := Boolean( Value );
      _Caption_XLabel:    (AObject as XLabel).Caption := PChar( Value );
      _Font_Name_XLabel:
        begin
          if StrEq( PChar( Value ), '' ) then
          begin
            DfltFont := XFont.Create;
            (AObject as XLabel).Font.Assign( DfltFont );
            DfltFont.Free;
          end
             else
            (AObject as XLabel).Font.Name := PChar( Value );
        end;
      _Font_Style_XLabel: (AObject as XLabel).Font.Style := Prop_FontStyleFrDWORD( Value );
      _Font_Height_XLabel:(AObject as XLabel).Font.Height := Integer( Value );
      _Font_Color_XLabel: (AObject as XLabel).Font.Color := XColor( Value );
      _Font_Orientation_XLabel: (AObject as XLabel).Font.Orientation := Integer( Value );
      _Font_Pitch_XLabel: (AObject as XLabel).Font.Pitch := XFontPitch( Value );
      _Font_Charset_XLabel: (AObject as XLabel).Font.Charset := Value;
      _Color_XLabel:      (AObject as XLabel).Color := XColor( Value );
      _VAlign:            (AObject as XLabel).VAlign := XVerticalAlignment( Value );
      end;
    cXCustomBevel:;
    cXBevel:
      case XBevel_Props( PropIdx ) of
      _BevelEdges:  (AObject as XBevel).BevelEdges := Prop_BevelEdgesFrDWORD( Value );
      _BevelInner:  (AObject as XBevel).BevelInner := XBevelStyle( Value );
      _BevelOuter:  (AObject as XBevel).BevelOuter := XBevelStyle( Value );
      _BevelWidth:  (AObject as XBevel).BevelWidth := Integer( Value );
      _BorderWidth: (AObject as XBevel).BorderWidth := Integer( Value );
      _Color_XBevel:(AObject as XBevel).Color := XColor( Value );
      end;
    cXPaint: // emulated by XBevel
      case XPaint_Props( PropIdx ) of
      _Color_XPaint:
        begin
          ((AObject as XBevel).Children[0] as XBevel).Color := XColor( Value );
          (AObject as XBevel).Color := Color2RGB( XColor( Value ) ) xor $FFFFFF;
        end;
      end;
    cNonVisual: // Pseudo-properties, invisible in Inspector
      case NonVisual_Props( PropIdx ) of
      _Left_NV: (AObject as XDsgnNonvisual).Position := XPoint( Integer( Value ),
                         (AObject as XDsgnNonVisual).Position.y );
      _Top_NV:  (AObject as XDsgnNonvisual).Position := XPoint(
                         (AObject as XDsgnNonVisual).Position.x,
                         Integer( Value ) );
      end;
    cZTimer: ;// all props are protected and designer maintains it virtually
    end;
  finally
  end;
end;

function Prop_ColorAsString( Value : DWORD): String;
begin
  case XColor(Value) of
  clScrollBar       : Result := 'clScrollBar';
  clBackground      : Result := 'clBackground';
  clActiveCaption   : Result := 'clActiveCaption';
  clInactiveCaption : Result := 'clInactiveCaption';
  clMenu            : Result := 'clMenu';
  clWindow          : Result := 'clWindow';
  clWindowFrame     : Result := 'clWindowFrame';
  clMenuText        : Result := 'clMenuText';
  clWindowText      : Result := 'clWindowText';
  clCaptionText     : Result := 'clCaptionText';
  clActiveBorder    : Result := 'clActiveBorder';
  clInactiveBorder  : Result := 'clInactiveBorder';
  clAppWorkSpace    : Result := 'clAppWorkSpace';
  clHighlight       : Result := 'clHighlight';
  clHighlightText   : Result := 'clHighlightText';
  clBtnFace         : Result := 'clBtnFace';
  clBtnShadow       : Result := 'clBtnShadow';
  clGrayText        : Result := 'clGrayText';
  clBtnText         : Result := 'clBtnText';
  clInactiveCaptionText: Result := 'clInactiveCaptionText';
  clBtnHighlight    : Result := 'clBtnHighlight';
  cl3DDkShadow      : Result := 'cl3DDkShadow';
  cl3DLight         : Result := 'cl3DLight';
  clInfoText        : Result := 'clInfoText';
  clInfoBk          : Result := 'clInfoBk';

  clBlack           : Result := 'clBlack';
  clMaroon          : Result := 'clMaroon';
  clGreen           : Result := 'clGreen';
  clOlive           : Result := 'clOlive';
  clNavy            : Result := 'clNavy';
  clPurple          : Result := 'clPurple';
  clTeal            : Result := 'clTeal';
  clGray            : Result := 'clGray';
  clSilver          : Result := 'clSilver';
  clRed             : Result := 'clRed';
  clLime            : Result := 'clLime';
  clYellow          : Result := 'clYellow';
  clBlue            : Result := 'clBlue';
  clFuchsia         : Result := 'clFuchsia';
  clAqua            : Result := 'clAqua';
  //clLtGray          : Result := 'clLtGray';
  //clDkGray          : Result := 'clDkGray';
  clWhite           : Result := 'clWhite';
  clNone            : Result := 'clNone';
  clDefault         : Result := 'clDefault';
  else
    Result := '$' + Int2Hex( DWord( Value ), 6 );
  end;
end;

function Prop_String2Color( const StrValue : string ) : DWORD;
const ColorValues : array[ 0..44 ] of XColor = (clScrollBar,clBackground,clActiveCaption,clInactiveCaption,
    clMenu,clWindow,clWindowFrame,clMenuText,clWindowText,clCaptionText,
    clActiveBorder,clInactiveBorder,clAppWorkSpace,clHighlight,clHighlightText,
    clBtnFace,clBtnShadow,clGrayText,clBtnText,clInactiveCaptionText,clBtnHighlight,
    cl3DDkShadow,cl3DLight,clInfoText,clInfoBk,clBlack,clMaroon,clGreen,clOlive,
    clNavy,clPurple,clTeal,clGray,clSilver,clRed,clLime,clYellow,clBlue,clFuchsia,
    clAqua,clLtGray,clDkGray,clWhite,clNone,clDefault);
var I, J, K : Integer;
begin
  if (StrValue[ 1 ] >= '0') and (StrValue[ 1 ] <= '9') then
  begin
    Result := Str2Int( StrValue );
    Exit;
  end;
  if StrValue[ 1 ] = '$' then
  begin
    Result := Hex2Int( CopyEnd( StrValue, 2 ) );
    Exit;
  end;
  I := 2; K := 0;
  while I < Length( Values_XColor ) do
  begin
    J := IndexOfChar( CopyEnd( Values_XColor, I ), ',' );
    if J <= 0 then break;
    J := I + J - 1;
    if StrEq( Copy( Values_XColor, I, J - I ), StrValue ) then
    begin
      Result := ColorValues[ K ];
      Exit;
    end;
    I := J + 1;
    Inc( K );
  end;
  Result := $FFFFFFFF;
  {
  if StrValue[ 1 ] = '$' then
     Result := Hex2Int( CopyEnd( StrValue, 2 ) )
  else
     Result := Str2Int( StrValue );
  }
end;

function GetClassesCount: Integer;
begin
  Result := Integer(High(TXCLClasses)) - Integer(Low(TXCLClasses)) + 1;
end;

function GetClassInfo(Idx: Integer): PXClassInfo;
begin
  Result := @ClassesData[ TXCLClasses( Idx ) ];
end;

function Prop_BevelEdgesAsDWORD( Value : XBevelEdges ) : DWORD;
begin
  Result := 0;
  if beLeft in Value then Result := 1;
  if beTop in Value then Result := Result or 2;
  if beRight in Value then Result := Result or 4;
  if beBottom in Value then Result := Result or 8;
end;

function Prop_BevelEdgesFrDWORD( Value : DWORD ) : XBevelEdges;
begin
  Result := [];
  if (Value and 1) <> 0 then Result := [beLeft];
  if (Value and 2) <> 0 then Result := Result + [beTop];
  if (Value and 4) <> 0 then Result := Result + [beRight];
  if (Value and 8) <> 0 then Result := Result + [beBottom];
end;

function Prop_FontStyleAsDWORD( Value : XFontStyle ) : DWORD;
begin
  Result := 0;
  if fsBold in Value then Result := 1;
  if fsItalic in Value then Result := Result or 2;
  if fsUnderline in Value then Result := Result or 4;
  if fsStrikeOut in Value then Result := Result or 8;
end;

function Prop_FontStyleFrDWORD( Value : DWORD ) : XFontStyle;
begin
  Result := [];
  if (Value and 1) <> 0 then Result := [fsBold];
  if (Value and 2) <> 0 then Result := Result + [fsItalic];
  if (Value and 4) <> 0 then Result := Result + [fsUnderline];
  if (Value and 8) <> 0 then Result := Result + [fsStrikeOut];
end;

/////////////////////////////////////////////////////////////////
function CheckDesignerVersion( DesignerVersion : Word ) : Boolean;
begin
  Result := DesignerVersion >= NeedDsgnVer;
end;

function GetXLibVersion : Word;
begin
  Result := XLibVersion;
end;

// Names listed below can not be assigned to objects of a form
// during renaming its. These are only names of fields and methods
// which can be found in XCustomForm and its ancestors. Other
// reserved words (Pascal) are defined in designer.
const
  FormReservedNames : array[ 0..136 ] of String = (
    // XCustomForm:
    'CreateCustomObjects', 'CreateForm', 'DestroyForm', 'OnMoving',
    // XForm:
    'FFocusChanging', 'FCurrentControl',
    'Paint', 'ExcludeUpdRect', 'ExcludeUpdRgn', 'IncludeUpdRgn', 'Create', 'Destroy',
    'CurrentControl', 'MouseCaptureControl', 'On_Message', 'OnMouseMove', 'OnMouseDown',
    'OnMouseUp', 'OnKeyDown', 'OnKeyUp', 'OnKeyPress', 'AvailableClientRect',
    'OnResize', 'GotoControl', 'AutoSize', 'AutoAlign', 'Margin', 'ControlAtPos',
    'ControlAtPosEx', 'CreateWindow', 'WndProc', 'GetWndUpdRgn', 'DoSetFocus',
    'ControlRect', 'OnPainted',
    // XWindow:
    'fDestroying', 'fMenu', 'SetEnabled', 'GetEnabled', 'GetVisible', 'SetVisible',
    'SetColor', 'GetBoundsRect', 'SetBoundsRect', 'GetIcon', 'GetWindowHandle',
    'CreateParams', 'CreateSubclass', 'Handle', 'Enabled', 'ParentWindow',
    'ClsStyle', 'Style', 'ExStyle', 'Cursor', 'Icon', 'Menu', 'Caption',
    'Visible', 'Show', 'Hide', 'Color', 'BoundsRect', 'Left', 'Top', 'Width', 'Height',
    'Position', 'OnClose', 'WindowState', 'ClientToScreen', 'ScreenToClient',
    'ClientRect', 'Canvas', 'Invalidate', 'Windowed', 'CallDefWndProc',
    'DoSetFocus', 'HandleAllocated',
    // XVisual:
    'fColor', 'FScrollOffset', 'fMembersCount', 'GetEnabled', 'SetEnabled',
    'GetVisible', 'SetVisible', 'GetBoundsRect', 'SetBoundsRect', 'PaintChild',
    'PaintChildren', 'PaintErase', 'GetWindowHandle', 'SetColor', 'Color',
    'DestroyChildren', 'GetParentWnd', 'CreateChildWindows', 'DoScrollOffset',
    'MembersCreating', 'MembersCreated', 'AdjustMembers', 'ElementAtPos',
    'Invalidate', 'ReadScrollOffset', 'MembersCount', 'Children',
    'ChildCount', 'Members',
    // XClass
    'fChildren', 'Parent', 'AddChild', 'DeleteChild', 'Tag',
    // TObject
    'AfterConstruction', 'BeforeDestruction', 'ClassInfo', 'ClassName', 'ClassNameIs',
    'ClassParent', 'ClassType', 'CleanupInstance', 'DefaultHandler',
    'Dispatch', 'FieldAddress', 'Free', 'FreeInstance', 'GetInterface',
    'GetInterfaceEntry', 'GetInterfaceTable', 'InheritsFrom', 'InitInstance',
    'InstanceSize', 'MethodAddress', 'MethodName', 'NewInstance', 'SafeCallException',
    ''
  );

function GetFormReservedNames : PStringArray;
begin
  Result := @FormReservedNames[ 0 ];
end;

procedure UnInstall;
begin
  Applet.Free;
  Applet := nil;
  MainAppl := nil;
  AppletTerminated := True;
  AppletRunning := False;
end;

function InitInterface( MainInst : HInst; MainApplet : XApplet ) : Pointer;
var Main : XForm;
    SaveInst : HInst;
begin
  MainInstance := hInstance;

  UseAligner;
  UseFontRotator;
  //UseBtnRepeats;

  SaveInst := hInstance;

  if Applet = nil then
  begin
    Applet := XApplet.Create;
    MainApplet := Applet;
    MainAppl := Applet;
    Applet.Icon := LoadIcon( SaveInst, 'WIZARDICON' );
    Applet.Caption := 'Library Applet';
    Applet.Visible := False;
    Applet.CreateAppButton;
  end;

  Main := XForm.Create( MainApplet );
  Main.Caption := 'Library Main';
  Main.BoundsRect := XRect( 400, 200, 450, 250 );
  Main.Visible := False;

  Main.CreateWindow;

  hInstance := SaveInst;

  Result := Applet;
  AppletRunning := True;
end;

function CreateForm( Visible : Boolean; Caption : PChar; Rect : PRect; ClsStyle, Style, ExStyle : DWORD;
                     Flags : DWORD; CloseEvnt : XOnEventAccept;
                     ActivateEvnt, SelectEvnt, DeselectEvnt, ResizeEvnt, MoveEvnt : XOnEvent;
                     ClickEvent : XOnDsgnClick; KeyDownEvent, KeyUpEvent : XOnDsgnKey )
                     : TObject;
var F : XDsgnForm;
begin
  F := XDsgnForm.Create( MainAppl );
  F.Visible := Visible;
  if (Flags and 1) <> 0 then
     F.ClsStyle := ClsStyle;
  if (Flags and 2) <> 0 then
     F.Style := Style;
  if (Flags and 4) <> 0 then
     F.ExStyle := ExStyle;
  F.Caption := Caption;
  if Rect <> nil then
  if not RectsEqual( Rect^, XRect( 0, 0, 0, 0 ) ) then
     F.BoundsRect := Rect^;
  F.OnClose := CloseEvnt;
  F.OnActivate := ActivateEvnt;
  F.OnMoving := MoveEvnt;
  F.OnResize := ResizeEvnt;
  F.OnMoveObject := MoveEvnt;
  F.OnResizeControl := ResizeEvnt;
  F.OnSelectObject := SelectEvnt;
  F.OnDeselectObject := DeselectEvnt;
  F.OnClick := ClickEvent;
  F.OnKeyDown := KeyDownEvent;
  F.OnKeyUp := KeyUpEvent;
  Result := F;
end;

function CreateFormWindow( Form : TObject ) : HWnd;
var F : XForm;
begin
  if not Applet.HandleAllocated then
     Applet.CreateAppButton;
  F := Form as XForm;
  if not F.HandleAllocated then
     F.CreateWindow;
  Result := F.Handle;
end;

procedure SelectObject( Form : TObject; Obj : TObject );
begin
  (Form as XDsgnForm).SelectObject( Obj );
end;

procedure DeselectObject( Form : TObject; Obj : TObject );
begin
  (Form as XDsgnForm).DeselectObj( Obj );
end;

function CreateNonvisual( FormObj : TObject; Visible : Boolean; ImgList : THandle;
                          ImgIdx : Integer; Pos : TPoint ) : TObject;
var NV : XDsgnNonVisual;
begin
  NV := XDsgnNonvisual.Create( FormObj as XDsgnForm, ImgList, ImgIDx, Pos );
  NV.Visible := Visible;
  Result := NV;
end;

function GetNonvisualWindow( NonVisual : TObject ) : HWnd;
begin
  Result := (NonVisual as XDsgnNonvisual).Handle;
end;

procedure SetNonVisualIcon( NonVisual : TObject; ImgList : THandle;
                            ImgIdx : Integer );
var NV : XDsgnNonvisual;
begin
  NV := NonVisual as XDsgnNonVisual;
  NV.ImgList := ImgList;
  NV.ImgIdx := ImgIdx;
end;

{
  Enumerating fonts entries.
}

var FontList : ZStrList;

{
  Incorrectly defined header of function EnumAllFonts caused crashes of system
  in Kernel32. Second parameter is by Win32 SDK pointer to data structure, but
  it was not defined as such parameter.
  When function header is corrected using VAR parameters, all is now working
  fine.
  
// Old "invalid" function header, here ------V is forgotten VAR parameter
function EnumAllFonts( LogFont: PEnumLogFont; FontPhysAttrs : TNewTextMetric;
                        FontType : Integer; LParam : LParam ) : Integer; stdcall;
}

function EnumAllFonts(var LogFont: TEnumLogFont;
  var FontPhysAttrs : TNewTextMetric; FontType : Integer; LParam : LParam) : Integer; stdcall;
var S : String;
    I : Integer;
begin
  Result := 1;
  S := LogFont.elfFullName;
  for I := 0 to FontList.Count - 1 do
  begin
    if StrEq( S, FontList[ I ] ) then Exit;
    if S < FontList[ I ] then
    begin
      FontList.Insert( I, S );
      Exit;
    end;
  end;
  FontList.Add( S );
end;

function GetFontsCount : Integer;
var LogFont : TLogFont;
    DC : HDC;
begin
  if FontList = nil then
  begin
    FontList := ZStrList.Create;
    FillChar( LogFont, SizeOf( LogFont ), 0 );
    DC := GetDC( 0 );
    EnumFontFamilies( DC, nil, @EnumAllFonts, 0 );
    ReleaseDC( 0, DC );
  end;
  Result := FontList.Count;
end;

function GetFontNames( Idx : Integer ) : String;
begin
  Result := '';
  if Idx >= GetFontsCount then Exit;
  Result := FontList[ Idx ];
end;

initialization
finalization
  FontList.Free;
  FontList := nil;

end.
